@@ -372,10 +372,10 @@ int m_asprintf(char **strp, const char *fmt, ...)
 
 /* print len bytes from buf in the format of well known "hd",
  * adjust displayed offset by file_offset */
-void fprintf_hex(FILE *fp, off_t file_offset, const void *buf, unsigned len)
+void fprintf_hex(FILE *fp, off_t file_offset, const void *buf, size_t len)
 {
 	const unsigned char *c = buf;
-	unsigned o;
+	size_t o;
 	int skipped = 0;
 
 	for (o = 0; o + 16 < len; o += 16, c += 16) {
@@ -396,7 +396,7 @@ void fprintf_hex(FILE *fp, off_t file_offset, const void *buf, unsigned len)
 			"  %02x %02x %02x %02x %02x %02x %02x %02x"
 			/* the same as printable char or '.' */
 			"  |%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c|\n",
-			(unsigned long long)o + file_offset,
+			(unsigned long long) o + file_offset,
 			c[0], c[1], c[2], c[3], c[4], c[5], c[6], c[7],
 			c[8], c[9], c[10], c[11], c[12], c[13], c[14], c[15],
 
@@ -411,22 +411,22 @@ void fprintf_hex(FILE *fp, off_t file_offset, const void *buf, unsigned len)
 		fprintf(fp, "*\n");
 	}
 	if (o < len) {
-		unsigned remaining = len - o;
-		unsigned i;
-		fprintf(fp, "%08llx ", (unsigned long long)o + file_offset);
+		size_t remaining = len - o;
+		size_t i;
+		fprintf(fp, "%08llx ", (unsigned long long) o + file_offset);
 		for (i = 0; i < remaining; i++) {
 			if (i == 8)
 				fprintf(fp, " ");
 			fprintf(fp, " %02x", c[i]);
 		}
-		fprintf(fp, "%*s  |", (16 - i)*3 + (i < 8), "");
+		fprintf(fp, "%*s  |", (int) ((16 - i)*3 + (i < 8)), "");
 		for (i = 0; i < remaining; i++)
 			fprintf(fp, "%c", p_(c[i]));
 #undef p
 #undef p_
 		fprintf(fp, "|\n");
 	}
-	fprintf(fp, "%08llx\n", (unsigned long long)len + file_offset);
+	fprintf(fp, "%08llx\n", (unsigned long long) len + file_offset);
 }
 
 
@@ -580,7 +580,14 @@ int get_fd_lockfile_timeout(const char *path, int seconds)
     };
 
     if ((fd = open(path, O_RDWR | O_CREAT, 0600)) < 0) {
-	fprintf(stderr,"open(%s): %m\n",path);
+	err = errno;
+	log_err("open(%s): %m\n", path);
+	if (err == EACCES) {
+		if (geteuid() != 0)
+			log_err("Try again with 'sudo' or as root?\n");
+		else
+			log_err("Maybe 'rm -f %s', or check sysctl fs.protected_regular\n");
+	}
 	return -1;
     }
 
@@ -674,6 +681,80 @@ int only_digits(const char *s)
 	return c != s && *c == 0;
 }
 
+void ensure_dir(const char *dirname)
+{
+	struct stat sb;
+	char *prefix;
+	char *s;
+
+	/* Nonsense usage. Handle followup failure elsewhere.
+	 * Won't happen, but make static analysers happy. */
+	if (!dirname || !*dirname)
+		return;
+
+	/* If we are not root,
+	 * do we even want to attempt to create the directory? */
+	if (geteuid() != 0)
+		return;
+
+	if (!stat(dirname, &sb) && S_ISDIR(sb.st_mode)) {
+		/* FIXME do we even want to check/enforce ownership? */
+		if (sb.st_uid != 0 /* && geteuid() == 0 */) {
+			int dfd = open(dirname, O_DIRECTORY,O_RDONLY);
+			if (dfd >= 0) {
+				if (fchown(dfd, 0, 0)) {
+					/* so I am root, but I cannot chown that directory? */
+					log_err("cannot chown root:root %s: %m\n", dirname);
+				}
+				close(dfd);
+			}
+		}
+		return;
+	}
+
+	prefix = strdupa(dirname);
+	if (NULL == prefix) {
+		perror("");
+		exit(20);
+	}
+
+	/* we could exit(20); on failure below, but let's try to continue and
+	 * produce the same error messages we would produce if we would just
+	 * try to use the directory without attempting to create it. */
+
+	/* recursively create directory */
+	s = prefix;
+	while ((s=strchrnul(s,'/'))) {
+		char slash_or_nul = *s;
+		*s = '\0';
+		if (*prefix) {
+			if (mkdir(prefix, S_IRWXU)) {
+				if (errno == EEXIST) {
+					if (stat(prefix,&sb)) {
+						log_err("stat(%s): %m\n", prefix);
+						return;
+					}
+					if (!S_ISDIR(sb.st_mode)) {
+						fprintf(stderr,
+							"Existing prefix %s is not a directory\n"
+							"Failed to create directory %s\n",
+							prefix, dirname);
+						return;
+					}
+				} else {
+					 /* Don't complain for read-only FS */
+					if (errno != EROFS)
+						log_err("Failed to create directory %s: %m\n", dirname);
+					return;
+				}
+			}
+		}
+		if (slash_or_nul == '\0')
+			break;
+		*s++ = '/';
+	}
+}
+
 int dt_lock_drbd(int minor)
 {
 	int sz, lfd;
@@ -692,11 +773,7 @@ int dt_lock_drbd(int minor)
 	 * We should store something in the meta data to detect such abuses.
 	 */
 
-	/* NOTE that /var/lock/drbd-*-* may not be "secure",
-	 * maybe we should rather use /var/lock/drbd/drbd-*-*,
-	 * and make sure that /var/lock/drbd is drwx.-..-. root:root  ...
-	 */
-
+	ensure_dir(drbd_lock_dir());
 	sz = asprintf(&lfname, "%s/drbd-%d-%d",
 		      drbd_lock_dir(), LANANA_DRBD_MAJOR, minor);
 	if (sz < 0) {
@@ -849,34 +926,40 @@ new_strtoll(const char *s, const char def_unit, unsigned long long *rv)
 	return MSE_OK;
 }
 
-struct err_state {
+struct logging_state {
+	unsigned int initialized:1;
 	unsigned int stderr_available:1;
 };
 
-static struct err_state err_state = { /* all zero */ };
+static struct logging_state logging_state = { /* all zero */ };
 
-/* Call initialize_err() before creating any FD */
-void initialize_err(void)
+/* Call initialize_logging() before creating any FD */
+void initialize_logging(void)
 {
 	int err;
 
+	if (logging_state.initialized)
+		return;
+
 	err = fcntl(STDERR_FILENO, F_GETFL);
 	if (err < 0 && errno == EBADF) {
-		err_state.stderr_available = 0;
+		logging_state.stderr_available = 0;
 		openlog(NULL, LOG_PID, LOG_SYSLOG);
 	} else {
-		err_state.stderr_available = 1;
+		logging_state.stderr_available = 1;
 	}
+	logging_state.initialized = 1;
 }
 
-int err(const char *format, ...)
+/* print to stderr or syslog */
+int log_err(const char *format, ...)
 {
 	va_list ap;
 	int n;
 
 	va_start(ap, format);
 
-	if (err_state.stderr_available) {
+	if (logging_state.stderr_available) {
 		n = vfprintf(stderr, format, ap);
 	} else {
 		vsyslog(LOG_ERR, format, ap);
@@ -910,12 +993,12 @@ const char *esc(char *str)
 				*e++ = '\\';
 			}
 			if (e - buffer >= 1022) {
-				err("string too long.\n");
+				log_err("string too long.\n");
 				exit(E_SYNTAX);
 			}
 			*e++ = *ue++;
 			if (e - buffer >= 1022) {
-				err("string too long.\n");
+				log_err("string too long.\n");
 				exit(E_SYNTAX);
 			}
 		}
@@ -964,6 +1047,6 @@ const char *esc_xml(char *str)
 	return str;
 
 too_long:
-	err("string too long.\n");
+	log_err("string too long.\n");
 	exit(E_SYNTAX);
 }
