@@ -1067,6 +1067,7 @@ struct context_def device_options_ctx = {
 	.fields = {
 		{ "max-bio-size", NUMERIC(max_bio_size, MAX_BIO_SIZE) },
 		{ "diskless", FLAG(intentional_diskless) },
+		{ "block-size", NUMERIC(block_size, BLOCK_SIZE) },
 		{ } },
 };
 
@@ -1138,6 +1139,19 @@ struct context_def adjust_ctx = {
 		{ } },
 };
 
+struct context_def status_ctx = {
+	.fields = {
+		{ "statistics", .argument_is_optional = true },
+		{ } },
+};
+
+struct context_def repair_md_ctx = {
+       .fields = {
+		{ .name = "tentative", .argument_is_optional = true },
+		{ .name = "force", .argument_is_optional = true },
+		{ } },
+};
+
 // only used by drbdadm's config file parser:
 struct context_def handlers_ctx = {
 	.fields = {
