@@ -96,6 +96,7 @@ struct d_volume
 	int meta_major;
 	int meta_minor;
 	STAILQ_ENTRY(d_volume) link;
+	struct options device_options;
 	struct options disk_options; /* Additional per volume options */
 	struct options pd_options; /* peer device options */
 	struct peer_devices peer_devices;
@@ -147,12 +148,14 @@ struct hname_address
 	char *name;			/* parsed */
 	int config_line;		/* parsed here */
 	struct d_address address;	/* parsed */
+	struct d_address outside_address;   /* parsed */
 	struct d_proxy_info *proxy;     /* parsed here */
 	struct d_host_info *host_info;	/* determined in post_parse */
 	unsigned int used_as_me:1;
 	unsigned int faked_hostname:1;
 	unsigned int by_address:1;
 	unsigned int parsed_address:1;
+	unsigned int parsed_outside_address:1;
 	unsigned int parsed_port:1;
 	unsigned int conflicts:1;
 	STAILQ_ENTRY(hname_address) link;
@@ -227,6 +230,7 @@ struct d_resource
 	struct meshes meshes;
 
 	struct options net_options; /* parsed here, inherited to connections */
+	struct options device_options;
 	struct options disk_options;
 	struct options pd_options; /* peer device options */
 	struct options res_options;
@@ -589,6 +593,7 @@ extern char *khelper_argv[];
 
 void print_platform_specific_versions(void);
 void assign_default_device(struct d_volume *vol);
+int is_same_disk(const char *a, const char *b);
 
 void generate_conf_file_locations(void);
 void free_conf_file_locations(void);
