@@ -50,6 +50,11 @@ YYSTYPE yylval;
 
 /////////////////////
 
+struct peer_delegate_data {
+	struct options *device_options;
+	struct options *peer_device_options;
+};
+
 static int c_section_start;
 static int parse_proxy_options(struct options *, struct options *);
 static void parse_skip(void);
@@ -147,7 +152,7 @@ void m_strtoll_range(const char *s, char def_unit,
 	unsigned long long r = m_strtoll(s, def_unit);
 	char unit[] = { def_unit != '1' ? def_unit : 0, 0 };
 	if (min > r || r > max) {
-		err("%s:%d: %s %s => %llu%s out of range [%llu..%llu]%s.\n",
+		log_err("%s:%d: %s %s => %llu%s out of range [%llu..%llu]%s.\n",
 			config_file, fline, name, s, r, unit, min, max, unit);
 		if (config_valid <= 1) {
 			config_valid = 0;
@@ -155,7 +160,7 @@ void m_strtoll_range(const char *s, char def_unit,
 		}
 	}
 	if (DEBUG_RANGE_CHECK) {
-		err("%s:%d: %s %s => %llu%s in range [%llu..%llu]%s.\n",
+		log_err("%s:%d: %s %s => %llu%s in range [%llu..%llu]%s.\n",
 			config_file, fline, name, s, r, unit, min, max, unit);
 	}
 }
@@ -192,7 +197,7 @@ void range_check(const enum range_checks what, const char *name,
 		M_STRTOLL_RANGE(NODE_ID);
 		break;
 	default:
-		err("%s:%d: unknown range for %s => %s\n", config_file, fline, name, value);
+		log_err("%s:%d: unknown range for %s => %s\n", config_file, fline, name, value);
 		break;
 	}
 }
@@ -201,7 +206,7 @@ struct d_option *new_opt(char *name, char *value)
 {
 	struct d_option *cn = calloc(1, sizeof(struct d_option));
 
-	/* err("%s:%d: %s = %s\n",config_file,line,name,value); */
+	/* log_err("%s:%d: %s = %s\n",config_file,line,name,value); */
 	cn->name = name;
 	cn->value = value;
 
@@ -211,14 +216,14 @@ struct d_option *new_opt(char *name, char *value)
 void pdperror(char *text)
 {
 	config_valid = 0;
-	err("%s:%d: in proxy plugin section: %s.\n", config_file, line, text);
+	log_err("%s:%d: in proxy plugin section: %s.\n", config_file, line, text);
 	exit(E_CONFIG_INVALID);
 }
 
 static void pperror(struct d_proxy_info *proxy, char *text)
 {
 	config_valid = 0;
-	err("%s:%d: in section: proxy on %s { ... } }: '%s' keyword missing.\n",
+	log_err("%s:%d: in section: proxy on %s { ... } }: '%s' keyword missing.\n",
 	    config_file, c_section_start, names_to_str(&proxy->on_hosts), text);
 }
 
@@ -297,19 +302,19 @@ int vcheck_uniq_file_line(
 
 	e = calloc(1, sizeof *e);
 	if (!e) {
-		err("calloc: %m\n");
+		log_err("calloc: %m\n");
 		exit(E_THINKO);
 	}
 
 	rv = vasprintf(&e->key, fmt, ap);
 
 	if (rv < 0) {
-		err("vasprintf: %m\n");
+		log_err("vasprintf: %m\n");
 		exit(E_THINKO);
 	}
 
 	if (EXIT_ON_CONFLICT && !what) {
-		err("Oops, unset argument in %s:%d.\n", __FILE__, __LINE__);
+		log_err("Oops, unset argument in %s:%d.\n", __FILE__, __LINE__);
 		exit(E_THINKO);
 	}
 	m_asprintf((char **)&e->data, "%s:%u", file, line);
@@ -318,7 +323,7 @@ int vcheck_uniq_file_line(
 	if (f) {
 		if (what) {
 			ENTRY *ep = *(ENTRY **)f;
-			err("%s: conflicting use of %s '%s' ...\n%s: %s '%s' first used here.\n",
+			log_err("%s: conflicting use of %s '%s' ...\n%s: %s '%s' first used here.\n",
 			    (char *)e->data, what, ep->key, (char *)ep->data, what, ep->key);
 		}
 		free(e->key);
@@ -328,7 +333,7 @@ int vcheck_uniq_file_line(
 	} else {
 		f = tsearch(e, bt, btree_key_cmp);
 		if (!f) {
-			err("tree entry (%s => %s) failed\n", e->key, (char *)e->data);
+			log_err("tree entry (%s => %s) failed\n", e->key, (char *)e->data);
 			exit(E_THINKO);
 		}
 		f = NULL;
@@ -346,7 +351,7 @@ int vcheck_uniq(void **bt, const char *what, const char *fmt, va_list ap)
 void pe_expected(const char *exp)
 {
 	const char *s = yytext;
-	err("%s:%u: Parse error: '%s' expected,\n\tbut got '%.20s%s'\n",
+	log_err("%s:%u: Parse error: '%s' expected,\n\tbut got '%.20s%s'\n",
 	    config_file, line, exp, s, strlen(s) > 20 ? "..." : "");
 	exit(E_CONFIG_INVALID);
 }
@@ -367,7 +372,7 @@ void check_string_error(int got)
 	default:
 		return;
 	}
-	err("%s:%u: %s >>>%.20s...<<<\n", config_file, line, msg, yytext);
+	log_err("%s:%u: %s >>>%.20s...<<<\n", config_file, line, msg, yytext);
 	exit(E_CONFIG_INVALID);
 }
 
@@ -378,7 +383,7 @@ void pe_expected_got(const char *exp, int got)
 	if (exp[0] == '\'' && exp[1] && exp[2] == '\'' && exp[3] == 0) {
 		tmp[0] = exp[1];
 	}
-	err("%s:%u: Parse error: '%s' expected,\n\tbut got '%.20s%s' (TK %d)\n",
+	log_err("%s:%u: Parse error: '%s' expected,\n\tbut got '%.20s%s' (TK %d)\n",
 	    config_file, line, tmp[0] ? tmp : exp, s, strlen(s) > 20 ? "..." : "", got);
 	exit(E_CONFIG_INVALID);
 }
@@ -388,7 +393,7 @@ static void parse_global(void)
 	fline = line;
 	check_uniq("global section", "global");
 	if (!STAILQ_EMPTY(&config)) {
-		err("%s:%u: You should put the global {} section\n\tin front of any resource {} section\n",
+		log_err("%s:%u: You should put the global {} section\n\tin front of any resource {} section\n",
 		    config_file, line);
 	}
 	EXP('{');
@@ -491,7 +496,7 @@ static void pe_valid_enums(const char **map, int nr_enums)
 	}
 
 	buffer[size - 3] = 0; /* Eliminate last " | " */
-	err("Allowed values are: %s\n", buffer);
+	log_err("Allowed values are: %s\n", buffer);
 }
 
 static void pe_field(const struct field_def *field, enum check_codes e, char *value)
@@ -505,7 +510,7 @@ static void pe_field(const struct field_def *field, enum check_codes e, char *va
 		[CC_STR_TOO_LONG] = "too long",
 		[CC_NOT_AN_ENUM_NUM] = "not valid",
 	};
-	err("%s:%u: Parse error: while parsing value ('%.20s%s')\nfor %s. Value is %s.\n",
+	log_err("%s:%u: Parse error: while parsing value ('%.20s%s')\nfor %s. Value is %s.\n",
 	    config_file, line,
 		value, strlen(value) > 20 ? "..." : "",
 		field->name, err_strings[e]);
@@ -513,7 +518,7 @@ static void pe_field(const struct field_def *field, enum check_codes e, char *va
 	if (e == CC_NOT_AN_ENUM)
 		pe_valid_enums(field->u.e.map, field->u.e.size);
 	else if (e == CC_STR_TOO_LONG)
-		err("max len: %u\n", field->u.s.max_len - 1);
+		log_err("max len: %u\n", field->u.s.max_len - 1);
 	/* else if (e == CC_NOT_AN_ENUM_NUM)
 	   pe_valid_enum_num((field->u.en.map, field->u.en.map_size); */
 
@@ -672,23 +677,35 @@ static struct options parse_options(struct context_def *options_def)
 
 static void insert_pd_options_delegate(void *ctx)
 {
-	struct options *options = ctx;
+	struct peer_delegate_data *params = ctx;
 	const struct field_def *field_def;
 	bool no_prefix;
 	char *value;
 
 	field_def = find_field(&no_prefix, &peer_device_options_ctx, yytext);
-	if (!field_def)
-		pe_options(&peer_device_options_ctx);
-	value = parse_option_value(field_def, no_prefix);
-	insert_tail(options, new_opt((char *)field_def->name, value));
+	if (field_def) {
+		value = parse_option_value(field_def, no_prefix);
+		insert_tail(params->peer_device_options, new_opt((char *)field_def->name, value));
+		return;
+	}
+	field_def = find_field(&no_prefix, &device_options_ctx, yytext);
+	if (field_def) {
+		value = parse_option_value(field_def, no_prefix);
+		insert_tail(params->device_options, new_opt((char *)field_def->name, value));
+		return;
+	}
+	pe_options(&peer_device_options_ctx); /* also mention device_options? */
 }
 
-static void parse_disk_options(struct options *disk_options, struct options *peer_device_options)
+static void parse_disk_options(struct options *disk_options,
+			       struct options *device_options,
+			       struct options *peer_device_options)
 {
+	struct peer_delegate_data params = {device_options, peer_device_options};
+
 	*disk_options = __parse_options(&attach_cmd_ctx,
-					insert_pd_options_delegate,
-					peer_device_options);
+					&insert_pd_options_delegate,
+					&params);
 }
 
 static void __parse_address(struct d_address *a)
@@ -844,7 +861,7 @@ void check_minor_nonsense(const char *devname, const int explicit_minor)
 		if (m == explicit_minor)
 			return;
 
-		err("%s:%d: explicit minor number must match with device name\n"
+		log_err("%s:%d: explicit minor number must match with device name\n"
 		    "\tTry \"device /dev/drbd%u minor %u;\",\n"
 		    "\tor leave off either device name or explicit minor.\n"
 		    "\tArbitrary device names must start with /dev/drbd_\n"
@@ -855,7 +872,7 @@ void check_minor_nonsense(const char *devname, const int explicit_minor)
 	} else if (devname[9] == '_')
 		return;
 
-	err("%s:%d: arbitrary device name must start with /dev/drbd_\n"
+	log_err("%s:%d: arbitrary device name must start with /dev/drbd_\n"
 	    "\tmind the '_'! (/dev/ is optional, but drbd_ is required)\n",
 	    config_file, fline);
 	config_valid = 0;
@@ -872,10 +889,11 @@ struct d_volume *alloc_volume(void)
 
 	vol = calloc(1, sizeof(struct d_volume));
 	if (vol == NULL) {
-		err("calloc: %m\n");
+		log_err("calloc: %m\n");
 		exit(E_EXEC_ERROR);
 	}
 
+	STAILQ_INIT(&vol->device_options);
 	STAILQ_INIT(&vol->disk_options);
 	STAILQ_INIT(&vol->pd_options);
 	STAILQ_INIT(&vol->peer_devices);
@@ -898,7 +916,7 @@ struct d_volume *volume0(struct volumes *volumes)
 			return vol;
 
 		config_valid = 0;
-		err("%s:%d: mixing explicit and implicit volumes is not allowed\n",
+		log_err("%s:%d: mixing explicit and implicit volumes is not allowed\n",
 		    config_file, line);
 		return vol;
 	}
@@ -918,7 +936,7 @@ int parse_volume_stmt(struct d_volume *vol, struct names* on_hosts, int token)
 			EXP(';');
 			break;
 		case '{':
-			parse_disk_options(&vol->disk_options, &vol->pd_options);
+			parse_disk_options(&vol->disk_options, &vol->device_options, &vol->pd_options);
 			break;
 		default:
 			check_string_error(token);
@@ -1000,7 +1018,7 @@ struct d_volume *parse_stacked_volume(int vnr)
 			break;
 		case TK_DISK:
 			EXP('{');
-			parse_disk_options(&vol->disk_options, &vol->pd_options);
+			parse_disk_options(&vol->disk_options, &vol->device_options, &vol->pd_options);
 			break;
 		case '}':
 			goto exit_loop;
@@ -1021,6 +1039,23 @@ enum parse_host_section_flags {
 	BY_ADDRESS  = 2,
 };
 
+static void fixup_name_inplace_for_strict_names(char *name)
+{
+	/* compare with DRBD module, drbd_nl.c:drbd_check_name_str() */
+	char *c;
+	for (c = name; *c; ++c) {
+		switch (*c) {
+		case '0' ... '9':
+		case 'A' ... 'Z':
+		case 'a' ... 'z':
+		case '+': case '-': case '.': case '_':
+			break;
+		default:
+			*c = '_';
+		}
+	}
+}
+
 static void parse_host_section(struct d_resource *res,
 			       struct names *on_hosts,
 			       enum parse_host_section_flags flags)
@@ -1048,9 +1083,11 @@ static void parse_host_section(struct d_resource *res,
 		host->by_address = 1;
 		__parse_address(&host->address);
 		if (!strcmp(host->address.af, "ipv6"))
-			m_asprintf(&fake_uname, "ipv6 [%s]:%s", host->address.addr, host->address.port);
+			m_asprintf(&fake_uname, "%s:%s", host->address.addr, host->address.port);
 		else
 			m_asprintf(&fake_uname, "%s:%s", host->address.addr, host->address.port);
+
+		fixup_name_inplace_for_strict_names(fake_uname);
 		insert_head(&host->on_hosts, names_from_str(fake_uname));
 
 		token = yylex();
@@ -1094,7 +1131,7 @@ static void parse_host_section(struct d_resource *res,
 			break;
 		case TK_ADDRESS:
 			if (host->by_address) {
-				err("%s:%d: address statement not allowed for floating {} host sections\n",
+				log_err("%s:%d: address statement not allowed for floating {} host sections\n",
 				    config_file, fline);
 				config_valid = 0;
 				exit(E_CONFIG_INVALID);
@@ -1159,7 +1196,7 @@ static void parse_skip()
 		}
 	}
 	if (!token) {
-		err("%s:%u: reached eof ""while parsing this skip block.\n",
+		log_err("%s:%u: reached eof ""while parsing this skip block.\n",
 		    config_file, fline);
 		exit(E_CONFIG_INVALID);
 	}
@@ -1230,7 +1267,7 @@ void startup_delegate(void *ctx)
 	struct d_resource *res = (struct d_resource *)ctx;
 
 	if (!strcmp(yytext, "become-primary-on")) {
-		/* err("Warn: Ignoring deprecated become-primary-on. Use automatic-promote\n"); */
+		/* log_err("Warn: Ignoring deprecated become-primary-on. Use automatic-promote\n"); */
 		int token;
 		do {
 			token = yylex();
@@ -1275,7 +1312,7 @@ void proxy_delegate(void *ctx)
 	opt = NULL;
 	token = yylex();
 	if (token != '{') {
-		err("%s:%d: expected \"{\" after \"proxy\" keyword\n",
+		log_err("%s:%d: expected \"{\" after \"proxy\" keyword\n",
 		    config_file, fline);
 		exit(E_CONFIG_INVALID);
 	}
@@ -1291,7 +1328,7 @@ void proxy_delegate(void *ctx)
 				if (STAILQ_EMPTY(&line))
 					goto out;
 
-				err("%s:%d: Missing \";\" before  \"}\"\n",
+				log_err("%s:%d: Missing \";\" before  \"}\"\n",
 				    config_file, fline);
 				exit(E_CONFIG_INVALID);
 			}
@@ -1347,6 +1384,25 @@ int parse_proxy_options_section(struct d_proxy_info **pp)
 	return parse_proxy_options(&proxy->options, &proxy->plugins);
 }
 
+static void parse_via(struct hname_address *ha)
+{
+	int token;
+
+	token = yylex();
+	switch (token) {
+	case TK_PROXY:
+		ha->proxy = parse_proxy_section();
+		break;
+	case TK_OUTSIDE_ADDRESS:
+		__parse_address(&ha->outside_address);
+		ha->parsed_outside_address = 1;
+		EXP(';');
+		break;
+	default:
+		pe_expected_got( "proxy | outside-address", token);
+	}
+}
+
 static struct hname_address *parse_hname_address_pair(struct path *path, int prev_token)
 {
 	struct hname_address *ha;
@@ -1391,16 +1447,12 @@ static struct hname_address *parse_hname_address_pair(struct path *path, int pre
 
 	parse_optional_via:
 		token = yylex();
+	case TK_VIA:
 		if (token == TK_VIA) {
-			EXP(TK_PROXY);
-			ha->proxy = parse_proxy_section();
+			parse_via(ha);
 		} else if (token != ';')
 			pe_expected_got( "via | ; ", token);
 		break;
-	case TK_VIA:
-		EXP(TK_PROXY);
-		ha->proxy = parse_proxy_section();
-		break;
 	case ';':
 		break;
 	default:
@@ -1416,7 +1468,7 @@ struct connection *alloc_connection()
 
 	conn = calloc(1, sizeof(struct connection));
 	if (conn == NULL) {
-		err("calloc: %m\n");
+		log_err("calloc: %m\n");
 		exit(E_EXEC_ERROR);
 	}
 	STAILQ_INIT(&conn->paths);
@@ -1438,7 +1490,7 @@ struct peer_device *alloc_peer_device()
 
 	peer_device = calloc(1, sizeof(*peer_device));
 	if (!peer_device)  {
-		err("calloc: %m\n");
+		log_err("calloc: %m\n");
 		exit(E_EXEC_ERROR);
 	}
 	STAILQ_INIT(&peer_device->pd_options);
@@ -1490,7 +1542,7 @@ struct path *alloc_path()
 
 	path = calloc(1, sizeof(struct path));
 	if (path == NULL) {
-		err("calloc: %m\n");
+		log_err("calloc: %m\n");
 		exit(E_EXEC_ERROR);
 	}
 	STAILQ_INIT(&path->hname_address_pairs);
@@ -1511,7 +1563,7 @@ static struct path *path0(struct connection *conn)
 	} else {
 		if (!path->implicit) {
 			config_valid = 0;
-			err("%s:%d: Explicit and implicit paths not allowed\n",
+			log_err("%s:%d: Explicit and implicit paths not allowed\n",
 			    config_file, line);
 		}
 	}
@@ -1536,7 +1588,7 @@ static struct path *parse_path()
 		case TK__REMOTE_HOST:
 			insert_tail(&path->hname_address_pairs, parse_hname_address_pair(path, token));
 			if (++hosts >= 3) {
-				err("%s:%d: only two 'host' keywords per path allowed\n",
+				log_err("%s:%d: only two 'host' keywords per path allowed\n",
 				    config_file, fline);
 				config_valid = 0;
 			}
@@ -1564,7 +1616,7 @@ static struct connection *parse_connection(enum pr_flags flags)
 	case '{':
 		break;
 	case TK_STRING:
-		err("%s:%d: explicitly named connections are deprecated!\n"
+		log_err("%s:%d: explicitly named connections are deprecated!\n"
 		    "\tits name ('%s') will be ignored\n", config_file, fline, yylval.txt);
 		EXP('{');
 		break;
@@ -1581,7 +1633,7 @@ static struct connection *parse_connection(enum pr_flags flags)
 			path = path0(conn);
 			insert_tail(&path->hname_address_pairs, parse_hname_address_pair(path, token));
 			if (++hosts >= 3) {
-				err("%s:%d: only two 'host' keywords per connection allowed\n",
+				log_err("%s:%d: only two 'host' keywords per connection allowed\n",
 				    config_file, fline);
 				config_valid = 0;
 			}
@@ -1591,7 +1643,7 @@ static struct connection *parse_connection(enum pr_flags flags)
 			break;
 		case TK_NET:
 			if (!STAILQ_EMPTY(&conn->net_options)) {
-				err("%s:%d: only one 'net' section per connection allowed\n",
+				log_err("%s:%d: only one 'net' section per connection allowed\n",
 				    config_file, fline);
 				config_valid = 0;
 			}
@@ -1621,7 +1673,7 @@ static struct connection *parse_connection(enum pr_flags flags)
 			break;
 		case '}':
 			if (STAILQ_EMPTY(&conn->paths) && !(flags & PARSE_FOR_ADJUST)) {
-				err("%s:%d: connection without a single path (maybe empty?) not allowed\n",
+				log_err("%s:%d: connection without a single path (maybe empty?) not allowed\n",
 						config_file, fline);
 				config_valid = 0;
 			}
@@ -1650,7 +1702,7 @@ void parse_connection_mesh(struct d_resource *res, enum pr_flags flags)
 			break;
 		case TK_NET:
 			if (!STAILQ_EMPTY(&mesh->net_options)) {
-				err("%s:%d: only one 'net' section allowed\n",
+				log_err("%s:%d: only one 'net' section allowed\n",
 				    config_file, fline);
 				config_valid = 0;
 			}
@@ -1686,6 +1738,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 	STAILQ_INIT(&res->connections);
 	STAILQ_INIT(&res->all_hosts);
 	STAILQ_INIT(&res->net_options);
+	STAILQ_INIT(&res->device_options);
 	STAILQ_INIT(&res->disk_options);
 	STAILQ_INIT(&res->pd_options);
 	STAILQ_INIT(&res->res_options);
@@ -1740,7 +1793,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 				break;
 			case '{':
 				check_upr("disk section", "%s:disk", res->name);
-				parse_disk_options(&options, &res->pd_options);
+				parse_disk_options(&options, &res->device_options, &res->pd_options);
 				STAILQ_CONCAT(&res->disk_options, &options);
 				break;
 			default:
@@ -1821,7 +1874,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 	if (flags == NO_HOST_SECT_ALLOWED && !STAILQ_EMPTY(&res->all_hosts)) {
 		config_valid = 0;
 
-		err("%s:%d: in the %s section, there are no host sections allowed.\n",
+		log_err("%s:%d: in the %s section, there are no host sections allowed.\n",
 		    config_file, c_section_start, res->name);
 	}
 
@@ -1836,7 +1889,7 @@ int was_file_already_seen(char *fn)
 
 	e = calloc(1, sizeof *e);
 	if (!e) {
-		err("calloc %m\n");
+		log_err("calloc %m\n");
 		exit(E_THINKO);
 	}
 
@@ -1844,7 +1897,7 @@ int was_file_already_seen(char *fn)
 	if (!real_path) {
 		real_path = strdup(fn);
 		if (!real_path) {
-			err("strdup %m");
+			log_err("strdup %m");
 			free(e);
 			exit(E_THINKO);
 		}
@@ -1860,7 +1913,7 @@ int was_file_already_seen(char *fn)
 	}
 
 	if (!tsearch(e, &global_btree, btree_key_cmp)) {
-		err("tree entry (%s => %s) failed\n", e->key, (char *)e->data);
+		log_err("tree entry (%s => %s) failed\n", e->key, (char *)e->data);
 		free(real_path);
 		free(e);
 		exit(E_THINKO);
@@ -1929,7 +1982,7 @@ static struct d_resource *template_file(const char *res_name)
 		restore_parse_context(&buffer);
 		fclose(f);
 	} else {
-		err("%s:%d: Failed to open template file '%s'.\n",
+		log_err("%s:%d: Failed to open template file '%s'.\n",
 		    config_save, line, file_name);
 		config_valid = 0;
 	}
@@ -1947,14 +2000,29 @@ void include_stmt(char *str)
 	size_t i;
 	int r;
 
-	cwd = pushd_to_current_config_file_unless_stdin();
-
-	/* """
+	/*
+	 * If the glob did not match any file,
+	 * there is nothing to do, silently ignore.
+	 * Unless it was no glob, but a literal,
+	 * which we would expect to exist.
+	 *
+	 * """
 	 * As a GNU extension, pglob->gl_flags is set to the
 	 * flags specified, ored with GLOB_MAGCHAR if any
 	 * metacharacters were found.
 	 * """
+	 *
+	 * But apparently |GLOB_MAGCHAR does not happen for GLOB_NOMATCH returns,
+	 * at least not consistently :-(
+	 * Also, there exist non-GNU libc
+	 * So we have this incomplete strchr heuristic anyways.
 	 */
+	bool contains_glob_magic_char =
+			strchr(str, '*') ||
+			strchr(str, '?') ||
+			strchr(str, '[');
+
+	cwd = pushd_to_current_config_file_unless_stdin();
 	r = glob(str, 0, NULL, &glob_buf);
 	if (r == 0) {
 		for (i=0; i<glob_buf.gl_pathc; i++) {
@@ -1965,41 +2033,31 @@ void include_stmt(char *str)
 			if (f) {
 				include_file(f, strdup(glob_buf.gl_pathv[i]));
 				fclose(f);
-			} else if (errno == ENOENT && glob_buf.gl_flags & GLOB_MAGCHAR) {
+			} else if (errno == ENOENT && contains_glob_magic_char) {
 				/* Noisily ignore race between glob expansion
 				 * and actual open. */
-				err("%s:%d: include file vanished after glob expansion '%s'.\n",
+				log_err("%s:%d: include file vanished after glob expansion '%s'.\n",
 				    config_save, line, glob_buf.gl_pathv[i]);
 				continue;
 			} else {
-				err("%s:%d: Failed to open include file '%s'.\n",
+				log_err("%s:%d: Failed to open include file '%s'.\n",
 				    config_save, line, glob_buf.gl_pathv[i]);
 				config_valid = 0;
 			}
 		}
 		globfree(&glob_buf);
 	} else if (r == GLOB_NOMATCH) {
-		/*
-		 * If the glob did not match any file,
-		 * there is nothing to do, silently ignore.
-		 * Unless it was no glob, but a literal,
-		 * which we would expect to exist.
-		 * Apparently |GLOB_MAGCHAR does not happen for GLOB_NOMATCH returns,
-		 * at least not consistently :-(
-		 * So we have this strchr heuristic anyways.
-		 */
-		/* if (!(glob_buf.gl_flags & GLOB_MAGCHAR)) { */
-		if (!strchr(str, '?') && !strchr(str, '*') && !strchr(str, '[')) {
-			err("%s:%d: Failed to open include file '%s'.\n",
+		if (!contains_glob_magic_char) {
+			log_err("%s:%d: Failed to open include file '%s'.\n",
 			    config_save, line, str);
 			config_valid = 0;
 		} else if (verbose) {
-			err("%s:%d: no match for include pattern '%s'.\n",
+			log_err("%s:%d: no match for include pattern '%s'.\n",
 			    config_save, line, str);
 		}
 		globfree(&glob_buf);
 	} else {
-		err("glob() failed: %d\n", r);
+		log_err("glob() failed: %d\n", r);
 		exit(E_USAGE);
 	}
 
@@ -2037,7 +2095,7 @@ static void validate_kmod(int token)
 #undef MOD_CMP
 
 	if (!kmod_valid) {
-		err("%s:%u: Validation error: drbd-module-version %d.%d.%d %s %s not true\n",
+		log_err("%s:%u: Validation error: drbd-module-version %d.%d.%d %s %s not true\n",
 				config_file, line,
 				have->version.major, have->version.minor, have->version.sublvl,
 				op, yylval.txt);
